@@ -139,6 +139,10 @@ process_module (Dwfl_Module *dwflmod,
 
 	/* get a debug context descriptor.*/
 	dwarf_info.dwarfd = dwfl_module_getdwarf (dwflmod, &dwbias);
+	if (dwarf_info.dwarfd == NULL) {
+		ERRMSG("dwfl_module_getdwarf error.\n");
+		return DWARF_CB_ABORT;
+	}
 	dwarf_info.elfd = dwarf_getelf(dwarf_info.dwarfd);
 
 	mod_name = dwfl_module_info(dwflmod, NULL, NULL, NULL, NULL, NULL,
@@ -940,7 +944,7 @@ get_debug_info(void)
 	elfd = dwarf_info.elfd;
 	dwarfd = dwarf_info.dwarfd;
 
-	if (elf_getshstrndx(elfd, &shstrndx) < 0) {
+	if (elf_getshdrstrndx(elfd, &shstrndx) < 0) {
 		ERRMSG("Can't get the section index of the string table.\n");
 		goto out;
 	}
@@ -1347,11 +1351,14 @@ get_die_nfields(unsigned long long die_off)
 	tag = dwarf_tag(die);
 	if (tag != DW_TAG_structure_type && tag != DW_TAG_union_type) {
 		ERRMSG("DIE is not of structure or union type.\n");
+		clean_dwfl_info();
 		return -1;
 	}
 
-	if (dwarf_child(die, &child) != 0)
+	if (dwarf_child(die, &child) != 0) {
+		clean_dwfl_info();
 		return -1;
+	}
 
 	/* Find the number of fields in the structure */
 	die = &child;
@@ -1363,6 +1370,7 @@ get_die_nfields(unsigned long long die_off)
 			continue;
 	} while (!dwarf_siblingof(die, die));
 
+	clean_dwfl_info();
 	return nfields;
 }
 
@@ -1373,6 +1381,7 @@ int
 get_die_member(unsigned long long die_off, int index, long *offset,
 		char **name, int *nbits, int *fbits, unsigned long long *m_die)
 {
+	const char *diename;
 	int tag, size, nfields = 0;
 	Dwarf_Die result, child, die_base, *die;
 
@@ -1388,11 +1397,14 @@ get_die_member(unsigned long long die_off, int index, long *offset,
 	tag = dwarf_tag(die);
 	if (tag != DW_TAG_structure_type && tag != DW_TAG_union_type) {
 		ERRMSG("DIE is not of structure or union type.\n");
+		clean_dwfl_info();
 		return -1;
 	}
 
-	if (dwarf_child(die, &child) != 0)
+	if (dwarf_child(die, &child) != 0) {
+		clean_dwfl_info();
 		return -1;
+	}
 
 	/* Find the correct field in the structure */
 	die = &child;
@@ -1408,6 +1420,7 @@ get_die_member(unsigned long long die_off, int index, long *offset,
 
 	if (nfields != index) {
 		ERRMSG("No member found at index %d.\n", index);
+		clean_dwfl_info();
 		return -1;
 	}
 
@@ -1415,7 +1428,14 @@ get_die_member(unsigned long long die_off, int index, long *offset,
 	if (!get_data_member_location(die, offset))
 		*offset = 0;
 
-	*name = dwarf_diename(die);
+	diename = dwarf_diename(die);
+	/*
+	 * Duplicate the string before we pass it to eppic layer. The
+	 * original string returned by dwarf layer will become invalid
+	 * after clean_dwfl_info() call.
+	 */
+	if (diename)
+		*name = strdup(diename);
 	*m_die = dwarf_dieoffset(die);
 
 	get_die_type(die, &die_base);
@@ -1432,6 +1452,7 @@ get_die_member(unsigned long long die_off, int index, long *offset,
 	 */
 	*nbits = *fbits = 0;
 
+	clean_dwfl_info();
 	if (size < 0)
 		return 0;
 	else
@@ -1455,21 +1476,36 @@ get_die_attr_type(unsigned long long die_off, int *type_flag,
 		return FALSE;
 	}
 
-	if (!get_die_type(&result, &result))
+	if (!get_die_type(&result, &result)) {
+		clean_dwfl_info();
 		return FALSE;
+	}
 
 	*die_attr_off = dwarf_dieoffset(&result);
 	*type_flag = dwarf_tag(&result);
+	clean_dwfl_info();
 	return TRUE;
 }
 
 /*
- * Get name attribute given the die offset
+ * Get name attribute given the die offset This function is called by eppic
+ * layer directly as one of the callback functions.
+ *
+ * This function returns a pointer to newly allocated string which is a
+ * duplicate of original string returned from dwarf APIs. The reason for doing
+ * this is because the original string returned by dwarf layer will become
+ * invalid (freed) as soon as we close the dwarf handle through
+ * clean_dwfl_info(). This avoids the segfault when caller (eppic layer) of
+ * this function tries to access the string pointer.
+ *
+ * NOTE: It is callers responsibility to free the memory of new string.
  */
 char *
 get_die_name(unsigned long long die_off)
 {
 	Dwarf_Die result;
+	char *name = NULL;
+	const char *diename;
 
 	if (!die_off)
 		return NULL;
@@ -1479,7 +1515,11 @@ get_die_name(unsigned long long die_off)
 		return NULL;
 	}
 
-	return dwarf_diename(&result);
+	diename = dwarf_diename(&result);
+	if (diename)
+		name = strdup(diename);
+	clean_dwfl_info();
+	return name;
 }
 
 /*
@@ -1510,6 +1550,7 @@ int
 get_die_length(unsigned long long die_off, int flag)
 {
 	Dwarf_Die result, die_base;
+	int size = 0;
 
 	if (!die_off)
 		return FALSE;
@@ -1519,17 +1560,22 @@ get_die_length(unsigned long long die_off, int flag)
 		return FALSE;
 	}
 
-	if (flag)
-		return dwarf_bytesize(&result);
+	if (flag) {
+		size = dwarf_bytesize(&result);
+		goto out;
+	}
 
 	get_die_type(&result, &die_base);
 	if (dwarf_tag(&die_base) == DW_TAG_array_type) {
 		dwarf_info.array_length = 0;
 		get_data_array_length(&result);
-		return dwarf_info.array_length;
+		size = dwarf_info.array_length;
 	} else {
-		return dwarf_bytesize(&die_base);
+		size = dwarf_bytesize(&die_base);
 	}
+out:
+	clean_dwfl_info();
+	return size;
 }
 
 /*
